@@ -88,7 +88,7 @@ int zip_set_permissions(mz_zip_archive *zip_archive, mz_uint file_index,
   version_by |= 23;
 
   external_attr &= 0x0000FFFF;
-  external_attr |= st.st_mode << 16;
+  external_attr |= (st.st_mode & 0777) << 16;
 
   if (! mz_zip_set_version_made_by(zip_archive, file_index, version_by) ||
       ! mz_zip_set_external_attr(zip_archive, file_index, external_attr)) {
@@ -113,6 +113,7 @@ int zip_get_permissions(mz_zip_archive_file_stat *stat, mode_t *mode) {
      we ignore them. */
   if (version_by != 3 || external_attr == 0) {
     *mode = stat->m_is_directory ? 0700 : 0600;
+    return 1;
   } else {
     *mode = (mode_t) external_attr & 0777;
   }
@@ -222,12 +223,17 @@ int zip_unzip(const char *czipfile, const char **cfiles, int num_files,
     }
 #ifndef _WIN32
     mode_t mode;
-    zip_get_permissions(&file_stat, &mode);
-    if (chmod(buffer, mode)) {
-      mz_zip_reader_end(&zip_archive);
-      if (buffer) free(buffer);
-      fclose(zfh);
-      ZIP_ERROR(R_ZIP_ESETPERM, key, czipfile);
+    /* returns 1 if there are no permissions. In that case we don't call
+       call chmod() and leave the permissions as they are, the file was
+       created with the default umask. */
+    int ret = zip_get_permissions(&file_stat, &mode);
+    if (!ret) {
+      if (chmod(buffer, mode)) {
+        mz_zip_reader_end(&zip_archive);
+        if (buffer) free(buffer);
+        fclose(zfh);
+        ZIP_ERROR(R_ZIP_ESETPERM, key, czipfile);
+      }
     }
 #endif
   }
@@ -285,7 +291,10 @@ int zip_zip(const char *czipfile, int num_files, const char **ckeys,
   if (cappend) {
     zfh = zip_open_utf8(czipfile, ZIP__APPEND, &filenameu16,
                         &filenameu16_len);
-    if (zfh == NULL) ZIP_ERROR(R_ZIP_EOPENAPPEND, czipfile);
+    if (zfh == NULL) {
+      if (filenameu16) free(filenameu16);
+      ZIP_ERROR(R_ZIP_EOPENAPPEND, czipfile);
+    }
     if (!mz_zip_reader_init_cfile(&zip_archive, zfh, 0, 0) ||
 	      !mz_zip_writer_init_from_reader(&zip_archive, NULL)) {
           if (filenameu16) free(filenameu16);
@@ -295,6 +304,10 @@ int zip_zip(const char *czipfile, int num_files, const char **ckeys,
   } else {
     zfh = zip_open_utf8(czipfile, ZIP__WRITE, &filenameu16,
                         &filenameu16_len);
+    if (zfh == NULL) {
+      if (filenameu16) free(filenameu16);
+      ZIP_ERROR(R_ZIP_EOPENWRITE, czipfile);
+    }
     if (!mz_zip_writer_init_cfile(&zip_archive, zfh, 0)) {
       if (filenameu16) free(filenameu16);
       fclose(zfh);
@@ -344,7 +357,6 @@ int zip_zip(const char *czipfile, int num_files, const char **ckeys,
       if (!ret) {
         if (filenameu16) free(filenameu16);
         mz_zip_writer_end(&zip_archive);
-        fclose(zfh);
         ZIP_ERROR(R_ZIP_EADDFILE, key, czipfile);
       }
     }
